# coding: utf-8
"""
    Kubernetes

    No description provided (generated by Swagger Codegen
    https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: v1.14.4

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import sys
import os
import re

# python 2 and python 3 compatibility library
from six import iteritems

from ..api_client import ApiClient


class StorageV1Api(object):
  """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

  def __init__(self, api_client=None):
    if api_client is None:
      api_client = ApiClient()
    self.api_client = api_client

  def create_storage_class(self, body, **kwargs):
    """
        create a StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_storage_class(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param V1StorageClass body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.create_storage_class_with_http_info(body, **kwargs)
    else:
      (data) = self.create_storage_class_with_http_info(body, **kwargs)
      return data

  def create_storage_class_with_http_info(self, body, **kwargs):
    """
        create a StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_storage_class_with_http_info(body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param V1StorageClass body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['body', 'pretty', 'dry_run', 'field_manager']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method create_storage_class' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `create_storage_class`'
      )

    collection_formats = {}

    path_params = {}

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/storageclasses',
        'POST',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1StorageClass',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def create_volume_attachment(self, body, **kwargs):
    """
        create a VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_volume_attachment(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param V1VolumeAttachment body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.create_volume_attachment_with_http_info(body, **kwargs)
    else:
      (data) = self.create_volume_attachment_with_http_info(body, **kwargs)
      return data

  def create_volume_attachment_with_http_info(self, body, **kwargs):
    """
        create a VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_volume_attachment_with_http_info(body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param V1VolumeAttachment body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['body', 'pretty', 'dry_run', 'field_manager']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method create_volume_attachment' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `create_volume_attachment`'
      )

    collection_formats = {}

    path_params = {}

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments',
        'POST',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachment',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def delete_collection_storage_class(self, **kwargs):
    """
        delete collection of StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_collection_storage_class(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.delete_collection_storage_class_with_http_info(**kwargs)
    else:
      (data) = self.delete_collection_storage_class_with_http_info(**kwargs)
      return data

  def delete_collection_storage_class_with_http_info(self, **kwargs):
    """
        delete collection of StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread =
        api.delete_collection_storage_class_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = [
        'pretty', '_continue', 'field_selector', 'label_selector', 'limit',
        'resource_version', 'timeout_seconds', 'watch'
    ]
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method delete_collection_storage_class' % key)
      params[key] = val
    del params['kwargs']

    collection_formats = {}

    path_params = {}

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if '_continue' in params:
      query_params.append(('continue', params['_continue']))
    if 'field_selector' in params:
      query_params.append(('fieldSelector', params['field_selector']))
    if 'label_selector' in params:
      query_params.append(('labelSelector', params['label_selector']))
    if 'limit' in params:
      query_params.append(('limit', params['limit']))
    if 'resource_version' in params:
      query_params.append(('resourceVersion', params['resource_version']))
    if 'timeout_seconds' in params:
      query_params.append(('timeoutSeconds', params['timeout_seconds']))
    if 'watch' in params:
      query_params.append(('watch', params['watch']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/storageclasses',
        'DELETE',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1Status',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def delete_collection_volume_attachment(self, **kwargs):
    """
        delete collection of VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_collection_volume_attachment(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.delete_collection_volume_attachment_with_http_info(**kwargs)
    else:
      (data) = self.delete_collection_volume_attachment_with_http_info(**kwargs)
      return data

  def delete_collection_volume_attachment_with_http_info(self, **kwargs):
    """
        delete collection of VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread =
        api.delete_collection_volume_attachment_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = [
        'pretty', '_continue', 'field_selector', 'label_selector', 'limit',
        'resource_version', 'timeout_seconds', 'watch'
    ]
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method delete_collection_volume_attachment' % key)
      params[key] = val
    del params['kwargs']

    collection_formats = {}

    path_params = {}

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if '_continue' in params:
      query_params.append(('continue', params['_continue']))
    if 'field_selector' in params:
      query_params.append(('fieldSelector', params['field_selector']))
    if 'label_selector' in params:
      query_params.append(('labelSelector', params['label_selector']))
    if 'limit' in params:
      query_params.append(('limit', params['limit']))
    if 'resource_version' in params:
      query_params.append(('resourceVersion', params['resource_version']))
    if 'timeout_seconds' in params:
      query_params.append(('timeoutSeconds', params['timeout_seconds']))
    if 'watch' in params:
      query_params.append(('watch', params['watch']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments',
        'DELETE',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1Status',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def delete_storage_class(self, name, **kwargs):
    """
        delete a StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_storage_class(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param V1DeleteOptions body:
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param int grace_period_seconds: The duration in seconds before the
        object should be deleted. Value must be non-negative integer. The value
        zero indicates delete immediately. If this value is nil, the default
        grace period for the specified type will be used. Defaults to a per
        object value if not specified. zero means delete immediately.
        :param bool orphan_dependents: Deprecated: please use the
        PropagationPolicy, this field will be deprecated in 1.7. Should the
        dependent objects be orphaned. If true/false, the \"orphan\" finalizer
        will be added to/removed from the object's finalizers list. Either this
        field or PropagationPolicy may be set, but not both.
        :param str propagation_policy: Whether and how garbage collection will
        be performed. Either this field or OrphanDependents may be set, but not
        both. The default policy is decided by the existing finalizer set in the
        metadata.finalizers and the resource-specific default policy. Acceptable
        values are: 'Orphan' - orphan the dependents; 'Background' - allow the
        garbage collector to delete the dependents in the background;
        'Foreground' - a cascading policy that deletes all dependents in the
        foreground.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.delete_storage_class_with_http_info(name, **kwargs)
    else:
      (data) = self.delete_storage_class_with_http_info(name, **kwargs)
      return data

  def delete_storage_class_with_http_info(self, name, **kwargs):
    """
        delete a StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_storage_class_with_http_info(name,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param V1DeleteOptions body:
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param int grace_period_seconds: The duration in seconds before the
        object should be deleted. Value must be non-negative integer. The value
        zero indicates delete immediately. If this value is nil, the default
        grace period for the specified type will be used. Defaults to a per
        object value if not specified. zero means delete immediately.
        :param bool orphan_dependents: Deprecated: please use the
        PropagationPolicy, this field will be deprecated in 1.7. Should the
        dependent objects be orphaned. If true/false, the \"orphan\" finalizer
        will be added to/removed from the object's finalizers list. Either this
        field or PropagationPolicy may be set, but not both.
        :param str propagation_policy: Whether and how garbage collection will
        be performed. Either this field or OrphanDependents may be set, but not
        both. The default policy is decided by the existing finalizer set in the
        metadata.finalizers and the resource-specific default policy. Acceptable
        values are: 'Orphan' - orphan the dependents; 'Background' - allow the
        garbage collector to delete the dependents in the background;
        'Foreground' - a cascading policy that deletes all dependents in the
        foreground.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = [
        'name', 'pretty', 'body', 'dry_run', 'grace_period_seconds',
        'orphan_dependents', 'propagation_policy'
    ]
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method delete_storage_class' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `delete_storage_class`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'grace_period_seconds' in params:
      query_params.append(
          ('gracePeriodSeconds', params['grace_period_seconds']))
    if 'orphan_dependents' in params:
      query_params.append(('orphanDependents', params['orphan_dependents']))
    if 'propagation_policy' in params:
      query_params.append(('propagationPolicy', params['propagation_policy']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/storageclasses/{name}',
        'DELETE',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1Status',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def delete_volume_attachment(self, name, **kwargs):
    """
        delete a VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_volume_attachment(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param V1DeleteOptions body:
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param int grace_period_seconds: The duration in seconds before the
        object should be deleted. Value must be non-negative integer. The value
        zero indicates delete immediately. If this value is nil, the default
        grace period for the specified type will be used. Defaults to a per
        object value if not specified. zero means delete immediately.
        :param bool orphan_dependents: Deprecated: please use the
        PropagationPolicy, this field will be deprecated in 1.7. Should the
        dependent objects be orphaned. If true/false, the \"orphan\" finalizer
        will be added to/removed from the object's finalizers list. Either this
        field or PropagationPolicy may be set, but not both.
        :param str propagation_policy: Whether and how garbage collection will
        be performed. Either this field or OrphanDependents may be set, but not
        both. The default policy is decided by the existing finalizer set in the
        metadata.finalizers and the resource-specific default policy. Acceptable
        values are: 'Orphan' - orphan the dependents; 'Background' - allow the
        garbage collector to delete the dependents in the background;
        'Foreground' - a cascading policy that deletes all dependents in the
        foreground.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.delete_volume_attachment_with_http_info(name, **kwargs)
    else:
      (data) = self.delete_volume_attachment_with_http_info(name, **kwargs)
      return data

  def delete_volume_attachment_with_http_info(self, name, **kwargs):
    """
        delete a VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_volume_attachment_with_http_info(name,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param V1DeleteOptions body:
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param int grace_period_seconds: The duration in seconds before the
        object should be deleted. Value must be non-negative integer. The value
        zero indicates delete immediately. If this value is nil, the default
        grace period for the specified type will be used. Defaults to a per
        object value if not specified. zero means delete immediately.
        :param bool orphan_dependents: Deprecated: please use the
        PropagationPolicy, this field will be deprecated in 1.7. Should the
        dependent objects be orphaned. If true/false, the \"orphan\" finalizer
        will be added to/removed from the object's finalizers list. Either this
        field or PropagationPolicy may be set, but not both.
        :param str propagation_policy: Whether and how garbage collection will
        be performed. Either this field or OrphanDependents may be set, but not
        both. The default policy is decided by the existing finalizer set in the
        metadata.finalizers and the resource-specific default policy. Acceptable
        values are: 'Orphan' - orphan the dependents; 'Background' - allow the
        garbage collector to delete the dependents in the background;
        'Foreground' - a cascading policy that deletes all dependents in the
        foreground.
        :return: V1Status
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = [
        'name', 'pretty', 'body', 'dry_run', 'grace_period_seconds',
        'orphan_dependents', 'propagation_policy'
    ]
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method delete_volume_attachment' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `delete_volume_attachment`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'grace_period_seconds' in params:
      query_params.append(
          ('gracePeriodSeconds', params['grace_period_seconds']))
    if 'orphan_dependents' in params:
      query_params.append(('orphanDependents', params['orphan_dependents']))
    if 'propagation_policy' in params:
      query_params.append(('propagationPolicy', params['propagation_policy']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments/{name}',
        'DELETE',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1Status',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def get_api_resources(self, **kwargs):
    """
        get available resources
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_api_resources(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: V1APIResourceList
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.get_api_resources_with_http_info(**kwargs)
    else:
      (data) = self.get_api_resources_with_http_info(**kwargs)
      return data

  def get_api_resources_with_http_info(self, **kwargs):
    """
        get available resources
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_api_resources_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: V1APIResourceList
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = []
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method get_api_resources' % key)
      params[key] = val
    del params['kwargs']

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/',
        'GET',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1APIResourceList',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def list_storage_class(self, **kwargs):
    """
        list or watch objects of kind StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_storage_class(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1StorageClassList
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.list_storage_class_with_http_info(**kwargs)
    else:
      (data) = self.list_storage_class_with_http_info(**kwargs)
      return data

  def list_storage_class_with_http_info(self, **kwargs):
    """
        list or watch objects of kind StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_storage_class_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1StorageClassList
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = [
        'pretty', '_continue', 'field_selector', 'label_selector', 'limit',
        'resource_version', 'timeout_seconds', 'watch'
    ]
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method list_storage_class' % key)
      params[key] = val
    del params['kwargs']

    collection_formats = {}

    path_params = {}

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if '_continue' in params:
      query_params.append(('continue', params['_continue']))
    if 'field_selector' in params:
      query_params.append(('fieldSelector', params['field_selector']))
    if 'label_selector' in params:
      query_params.append(('labelSelector', params['label_selector']))
    if 'limit' in params:
      query_params.append(('limit', params['limit']))
    if 'resource_version' in params:
      query_params.append(('resourceVersion', params['resource_version']))
    if 'timeout_seconds' in params:
      query_params.append(('timeoutSeconds', params['timeout_seconds']))
    if 'watch' in params:
      query_params.append(('watch', params['watch']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf', 'application/json;stream=watch', 'application/vnd.kubernetes.protobuf;stream=watch'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/storageclasses',
        'GET',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1StorageClassList',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def list_volume_attachment(self, **kwargs):
    """
        list or watch objects of kind VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_volume_attachment(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1VolumeAttachmentList
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.list_volume_attachment_with_http_info(**kwargs)
    else:
      (data) = self.list_volume_attachment_with_http_info(**kwargs)
      return data

  def list_volume_attachment_with_http_info(self, **kwargs):
    """
        list or watch objects of kind VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_volume_attachment_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pretty: If 'true', then the output is pretty printed.
        :param str _continue: The continue option should be set when retrieving
        more results from the server. Since this value is server defined,
        clients may only use the continue value from a previous query result
        with identical query parameters (except for the value of continue) and
        the server may reject a continue value it does not recognize. If the
        specified continue value is no longer valid whether due to expiration
        (generally five to fifteen minutes) or a configuration change on the
        server, the server will respond with a 410 ResourceExpired error
        together with a continue token. If the client needs a consistent list,
        it must restart their list without the continue field. Otherwise, the
        client may send another list request with the token received with the
        410 error, the server will respond with a list starting from the next
        key, but from the latest snapshot, which is inconsistent from the
        previous list results - objects that are created, modified, or deleted
        after the first list request will be included in the response, as long
        as their keys are after the \"next key\".  This field is not supported
        when watch is true. Clients may start a watch from the last
        resourceVersion value returned by the server and not miss any
        modifications.
        :param str field_selector: A selector to restrict the list of returned
        objects by their fields. Defaults to everything.
        :param str label_selector: A selector to restrict the list of returned
        objects by their labels. Defaults to everything.
        :param int limit: limit is a maximum number of responses to return for a
        list call. If more items exist, the server will set the `continue` field
        on the list metadata to a value that can be used with the same initial
        query to retrieve the next set of results. Setting a limit may return
        fewer than the requested amount of items (up to zero items) in the event
        all requested objects are filtered out and clients should only use the
        presence of the continue field to determine whether more results are
        available. Servers may choose not to support the limit argument and will
        return all of the available results. If limit is specified and the
        continue field is empty, clients may assume that no more results are
        available. This field is not supported if watch is true.  The server
        guarantees that the objects returned when using continue will be
        identical to issuing a single list call without a limit - that is, no
        objects created, modified, or deleted after the first request is issued
        will be included in any subsequent continued requests. This is sometimes
        referred to as a consistent snapshot, and ensures that a client that is
        using limit to receive smaller chunks of a very large result can ensure
        they see all possible objects. If objects are updated during a chunked
        list the version of the object that was present at the time the first
        list result was calculated is returned.
        :param str resource_version: When specified with a watch call, shows
        changes that occur after that particular version of a resource. Defaults
        to changes from the beginning of history. When specified for list: - if
        unset, then the result is returned from remote storage based on
        quorum-read flag; - if it's 0, then we simply return what we currently
        have in cache, no guarantee; - if set to non zero, then the result is at
        least as fresh as given rv.
        :param int timeout_seconds: Timeout for the list/watch call. This limits
        the duration of the call, regardless of any activity or inactivity.
        :param bool watch: Watch for changes to the described resources and
        return them as a stream of add, update, and remove notifications.
        Specify resourceVersion.
        :return: V1VolumeAttachmentList
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = [
        'pretty', '_continue', 'field_selector', 'label_selector', 'limit',
        'resource_version', 'timeout_seconds', 'watch'
    ]
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method list_volume_attachment' % key)
      params[key] = val
    del params['kwargs']

    collection_formats = {}

    path_params = {}

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if '_continue' in params:
      query_params.append(('continue', params['_continue']))
    if 'field_selector' in params:
      query_params.append(('fieldSelector', params['field_selector']))
    if 'label_selector' in params:
      query_params.append(('labelSelector', params['label_selector']))
    if 'limit' in params:
      query_params.append(('limit', params['limit']))
    if 'resource_version' in params:
      query_params.append(('resourceVersion', params['resource_version']))
    if 'timeout_seconds' in params:
      query_params.append(('timeoutSeconds', params['timeout_seconds']))
    if 'watch' in params:
      query_params.append(('watch', params['watch']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf', 'application/json;stream=watch', 'application/vnd.kubernetes.protobuf;stream=watch'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments',
        'GET',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachmentList',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def patch_storage_class(self, name, body, **kwargs):
    """
        partially update the specified StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_storage_class(name, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param object body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint. This field is
        required for apply requests (application/apply-patch) but optional for
        non-apply patch types (JsonPatch, MergePatch, StrategicMergePatch).
        :param bool force: Force is going to \"force\" Apply requests. It means
        user will re-acquire conflicting fields owned by other people. Force
        flag must be unset for non-apply patch requests.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.patch_storage_class_with_http_info(name, body, **kwargs)
    else:
      (data) = self.patch_storage_class_with_http_info(name, body, **kwargs)
      return data

  def patch_storage_class_with_http_info(self, name, body, **kwargs):
    """
        partially update the specified StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_storage_class_with_http_info(name, body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param object body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint. This field is
        required for apply requests (application/apply-patch) but optional for
        non-apply patch types (JsonPatch, MergePatch, StrategicMergePatch).
        :param bool force: Force is going to \"force\" Apply requests. It means
        user will re-acquire conflicting fields owned by other people. Force
        flag must be unset for non-apply patch requests.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'body', 'pretty', 'dry_run', 'field_manager', 'force']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method patch_storage_class' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `patch_storage_class`'
      )
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `patch_storage_class`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))
    if 'force' in params:
      query_params.append(('force', params['force']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['application/json-patch+json', 'application/merge-patch+json', 'application/strategic-merge-patch+json'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/storageclasses/{name}',
        'PATCH',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1StorageClass',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def patch_volume_attachment(self, name, body, **kwargs):
    """
        partially update the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_volume_attachment(name, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param object body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint. This field is
        required for apply requests (application/apply-patch) but optional for
        non-apply patch types (JsonPatch, MergePatch, StrategicMergePatch).
        :param bool force: Force is going to \"force\" Apply requests. It means
        user will re-acquire conflicting fields owned by other people. Force
        flag must be unset for non-apply patch requests.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.patch_volume_attachment_with_http_info(name, body, **kwargs)
    else:
      (data) = self.patch_volume_attachment_with_http_info(name, body, **kwargs)
      return data

  def patch_volume_attachment_with_http_info(self, name, body, **kwargs):
    """
        partially update the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_volume_attachment_with_http_info(name, body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param object body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint. This field is
        required for apply requests (application/apply-patch) but optional for
        non-apply patch types (JsonPatch, MergePatch, StrategicMergePatch).
        :param bool force: Force is going to \"force\" Apply requests. It means
        user will re-acquire conflicting fields owned by other people. Force
        flag must be unset for non-apply patch requests.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'body', 'pretty', 'dry_run', 'field_manager', 'force']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method patch_volume_attachment' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `patch_volume_attachment`'
      )
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `patch_volume_attachment`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))
    if 'force' in params:
      query_params.append(('force', params['force']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['application/json-patch+json', 'application/merge-patch+json', 'application/strategic-merge-patch+json'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments/{name}',
        'PATCH',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachment',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def patch_volume_attachment_status(self, name, body, **kwargs):
    """
        partially update status of the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_volume_attachment_status(name, body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param object body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint. This field is
        required for apply requests (application/apply-patch) but optional for
        non-apply patch types (JsonPatch, MergePatch, StrategicMergePatch).
        :param bool force: Force is going to \"force\" Apply requests. It means
        user will re-acquire conflicting fields owned by other people. Force
        flag must be unset for non-apply patch requests.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.patch_volume_attachment_status_with_http_info(
          name, body, **kwargs)
    else:
      (data) = self.patch_volume_attachment_status_with_http_info(
          name, body, **kwargs)
      return data

  def patch_volume_attachment_status_with_http_info(self, name, body, **kwargs):
    """
        partially update status of the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_volume_attachment_status_with_http_info(name,
        body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param object body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint. This field is
        required for apply requests (application/apply-patch) but optional for
        non-apply patch types (JsonPatch, MergePatch, StrategicMergePatch).
        :param bool force: Force is going to \"force\" Apply requests. It means
        user will re-acquire conflicting fields owned by other people. Force
        flag must be unset for non-apply patch requests.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'body', 'pretty', 'dry_run', 'field_manager', 'force']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method patch_volume_attachment_status' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `patch_volume_attachment_status`'
      )
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `patch_volume_attachment_status`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))
    if 'force' in params:
      query_params.append(('force', params['force']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['application/json-patch+json', 'application/merge-patch+json', 'application/strategic-merge-patch+json'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments/{name}/status',
        'PATCH',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachment',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def read_storage_class(self, name, **kwargs):
    """
        read the specified StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.read_storage_class(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param bool exact: Should the export be exact.  Exact export maintains
        cluster-specific fields like 'Namespace'. Deprecated. Planned for
        removal in 1.18.
        :param bool export: Should this value be exported.  Export strips fields
        that a user can not specify. Deprecated. Planned for removal in 1.18.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.read_storage_class_with_http_info(name, **kwargs)
    else:
      (data) = self.read_storage_class_with_http_info(name, **kwargs)
      return data

  def read_storage_class_with_http_info(self, name, **kwargs):
    """
        read the specified StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.read_storage_class_with_http_info(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param bool exact: Should the export be exact.  Exact export maintains
        cluster-specific fields like 'Namespace'. Deprecated. Planned for
        removal in 1.18.
        :param bool export: Should this value be exported.  Export strips fields
        that a user can not specify. Deprecated. Planned for removal in 1.18.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'pretty', 'exact', 'export']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method read_storage_class' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `read_storage_class`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'exact' in params:
      query_params.append(('exact', params['exact']))
    if 'export' in params:
      query_params.append(('export', params['export']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/storageclasses/{name}',
        'GET',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1StorageClass',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def read_volume_attachment(self, name, **kwargs):
    """
        read the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.read_volume_attachment(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param bool exact: Should the export be exact.  Exact export maintains
        cluster-specific fields like 'Namespace'. Deprecated. Planned for
        removal in 1.18.
        :param bool export: Should this value be exported.  Export strips fields
        that a user can not specify. Deprecated. Planned for removal in 1.18.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.read_volume_attachment_with_http_info(name, **kwargs)
    else:
      (data) = self.read_volume_attachment_with_http_info(name, **kwargs)
      return data

  def read_volume_attachment_with_http_info(self, name, **kwargs):
    """
        read the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.read_volume_attachment_with_http_info(name,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param bool exact: Should the export be exact.  Exact export maintains
        cluster-specific fields like 'Namespace'. Deprecated. Planned for
        removal in 1.18.
        :param bool export: Should this value be exported.  Export strips fields
        that a user can not specify. Deprecated. Planned for removal in 1.18.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'pretty', 'exact', 'export']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method read_volume_attachment' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `read_volume_attachment`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'exact' in params:
      query_params.append(('exact', params['exact']))
    if 'export' in params:
      query_params.append(('export', params['export']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments/{name}',
        'GET',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachment',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def read_volume_attachment_status(self, name, **kwargs):
    """
        read status of the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.read_volume_attachment_status(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.read_volume_attachment_status_with_http_info(name, **kwargs)
    else:
      (data) = self.read_volume_attachment_status_with_http_info(name, **kwargs)
      return data

  def read_volume_attachment_status_with_http_info(self, name, **kwargs):
    """
        read status of the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.read_volume_attachment_status_with_http_info(name,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'pretty']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method read_volume_attachment_status' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `read_volume_attachment_status`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments/{name}/status',
        'GET',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachment',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def replace_storage_class(self, name, body, **kwargs):
    """
        replace the specified StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.replace_storage_class(name, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param V1StorageClass body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.replace_storage_class_with_http_info(name, body, **kwargs)
    else:
      (data) = self.replace_storage_class_with_http_info(name, body, **kwargs)
      return data

  def replace_storage_class_with_http_info(self, name, body, **kwargs):
    """
        replace the specified StorageClass
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.replace_storage_class_with_http_info(name, body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the StorageClass (required)
        :param V1StorageClass body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1StorageClass
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'body', 'pretty', 'dry_run', 'field_manager']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method replace_storage_class' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `replace_storage_class`'
      )
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `replace_storage_class`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/storageclasses/{name}',
        'PUT',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1StorageClass',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def replace_volume_attachment(self, name, body, **kwargs):
    """
        replace the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.replace_volume_attachment(name, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param V1VolumeAttachment body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.replace_volume_attachment_with_http_info(name, body, **kwargs)
    else:
      (data) = self.replace_volume_attachment_with_http_info(
          name, body, **kwargs)
      return data

  def replace_volume_attachment_with_http_info(self, name, body, **kwargs):
    """
        replace the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.replace_volume_attachment_with_http_info(name, body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param V1VolumeAttachment body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'body', 'pretty', 'dry_run', 'field_manager']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method replace_volume_attachment' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `replace_volume_attachment`'
      )
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `replace_volume_attachment`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments/{name}',
        'PUT',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachment',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)

  def replace_volume_attachment_status(self, name, body, **kwargs):
    """
        replace status of the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.replace_volume_attachment_status(name, body,
        async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param V1VolumeAttachment body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.replace_volume_attachment_status_with_http_info(
          name, body, **kwargs)
    else:
      (data) = self.replace_volume_attachment_status_with_http_info(
          name, body, **kwargs)
      return data

  def replace_volume_attachment_status_with_http_info(self, name, body,
                                                      **kwargs):
    """
        replace status of the specified VolumeAttachment
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.replace_volume_attachment_status_with_http_info(name,
        body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: name of the VolumeAttachment (required)
        :param V1VolumeAttachment body: (required)
        :param str pretty: If 'true', then the output is pretty printed.
        :param str dry_run: When present, indicates that modifications should
        not be persisted. An invalid or unrecognized dryRun directive will
        result in an error response and no further processing of the request.
        Valid values are: - All: all dry run stages will be processed
        :param str field_manager: fieldManager is a name associated with the
        actor or entity that is making these changes. The value must be less
        than or 128 characters long, and only contain printable characters, as
        defined by https://golang.org/pkg/unicode/#IsPrint.
        :return: V1VolumeAttachment
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = ['name', 'body', 'pretty', 'dry_run', 'field_manager']
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method replace_volume_attachment_status' % key)
      params[key] = val
    del params['kwargs']
    # verify the required parameter 'name' is set
    if ('name' not in params) or (params['name'] is None):
      raise ValueError(
          'Missing the required parameter `name` when calling `replace_volume_attachment_status`'
      )
    # verify the required parameter 'body' is set
    if ('body' not in params) or (params['body'] is None):
      raise ValueError(
          'Missing the required parameter `body` when calling `replace_volume_attachment_status`'
      )

    collection_formats = {}

    path_params = {}
    if 'name' in params:
      path_params['name'] = params['name']

    query_params = []
    if 'pretty' in params:
      query_params.append(('pretty', params['pretty']))
    if 'dry_run' in params:
      query_params.append(('dryRun', params['dry_run']))
    if 'field_manager' in params:
      query_params.append(('fieldManager', params['field_manager']))

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    if 'body' in params:
      body_params = params['body']
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['*/*'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/storage.k8s.io/v1/volumeattachments/{name}/status',
        'PUT',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1VolumeAttachment',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""cloud-shell ssh command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import datetime
import threading
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.cloud_shell import util
from googlecloudsdk.command_lib.util.ssh import ssh
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
import six


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
class Ssh(base.Command):
  """Allows you to establish an interactive SSH session with Cloud Shell."""

  detailed_help = {
      'DESCRIPTION':
          """\
        *{command}* lets you remotely log in to Cloud Shell. If your Cloud Shell
        is not currently running, this will cause it to be started before
        establishing the SSH session.
        """,
      'EXAMPLES':
          """\
        To SSH into your Cloud Shell, run:

            $ {command}

        To run a remote command in your Cloud Shell, run:

            $ {command} --command=ls
        """,
  }

  @staticmethod
  def Args(parser):
    util.ParseCommonArgs(parser)
    parser.add_argument(
        '--command',
        help="""\
        A command to run in Cloud Shell.

        Runs the command in Cloud Shell and then exits.
        """)
    parser.add_argument(
        '--dry-run',
        help="""\
        If provided, prints the command that would be run to standard out
        instead of executing it.
        """,
        action='store_true')
    parser.add_argument(
        '--ssh-flag',
        help='Additional flags to be passed to *ssh(1)*.',
        action='append')
    parser.add_argument(
        '--authorize-session',
        help="""\
        If provided, sends OAuth credentials to the current Cloud Shell session
        on behalf of the user. When this completes, the session will be
        authorized to run various Google Cloud command-line tools without
        requiring the user to manually authenticate.
        """,
        action='store_true')

  def Run(self, args):
    if not args.authorize_session:
      log.Print('Automatic authentication with GCP CLI tools in Cloud Shell is '
                'disabled. To enable, please rerun command with '
                '`--authorize-session` flag.')
    command_list = args.command.split(' ') if args.command else ['bash -l']
    project = properties.VALUES.core.project.Get()
    connection_info = util.PrepareEnvironment(args)
    if args.authorize_session:
      util.AuthorizeEnvironment()
    command = ssh.SSHCommand(
        remote=ssh.Remote(host=connection_info.host, user=connection_info.user),
        port=six.text_type(connection_info.port),
        identity_file=connection_info.key,
        remote_command=(['DEVSHELL_PROJECT_ID=' + project] if project else []) +
        command_list,
        extra_flags=args.ssh_flag,
        tty=not args.command,
        options={'StrictHostKeyChecking': 'no'},
    )

    if args.dry_run:
      elems = command.Build(connection_info.ssh_env)
      log.Print(' '.join([six.moves.shlex_quote(elem) for elem in elems]))
    elif args.authorize_session:
      self.done = threading.Event()
      thread = threading.Thread(target=self.Reauthorize, args=())
      thread.daemon = True
      thread.start()
      command.Run(connection_info.ssh_env)
      self.done.set()
    else:
      command.Run(connection_info.ssh_env)

  def Reauthorize(self):
    while not self.done.is_set():
      self.done.wait(
          (util.MIN_CREDS_EXPIRY - datetime.timedelta(minutes=2)).seconds)
      if not self.done.is_set():
        util.AuthorizeEnvironment()


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class SshAlpha(base.Command):
  """Allows you to establish an interactive SSH session with Cloud Shell."""

  detailed_help = {
      'DESCRIPTION':
          """\
        *{command}* lets you remotely log in to Cloud Shell. If your Cloud Shell
        is not currently running, this will cause it to be started before
        establishing the SSH session.
        """,
      'EXAMPLES':
          """\
        To SSH into your Cloud Shell, run:

            $ {command}

        To run a remote command in your Cloud Shell, run:

            $ {command} --command=ls
        """,
  }

  @staticmethod
  def Args(parser):
    util.ParseCommonArgs(parser)
    util.AddSshArgFlag(parser)
    parser.add_argument(
        '--command',
        help="""\
        A command to run in Cloud Shell.

        Runs the command in Cloud Shell and then exits.
        """)
    parser.add_argument(
        '--dry-run',
        help="""\
        If provided, prints the command that would be run to standard out
        instead of executing it.
        """,
        action='store_true')
    parser.add_argument(
        '--ssh-flag',
        help='Additional flags to be passed to *ssh(1)*.',
        action='append')
    parser.add_argument(
        '--authorize-session',
        help="""\
        If provided, sends OAuth credentials to the current Cloud Shell session
        on behalf of the user. When this completes, the session will be
        authorized to run various Google Cloud command-line tools without
        requiring the user to manually authenticate.
        """,
        action='store_true')

  def Run(self, args):
    if not args.authorize_session:
      log.Print(
          'Automatic authentication with GCP CLI tools in Cloud Shell is '
          'disabled. To enable, please rerun command with '
          '`--authorize-session` flag.')
    command_list = args.command.split(' ') if args.command else ['bash -l']
    project = properties.VALUES.core.project.Get()
    connection_info = util.PrepareEnvironment(args)
    if args.authorize_session:
      util.AuthorizeEnvironment()
    command = ssh.SSHCommand(
        remote=ssh.Remote(host=connection_info.host, user=connection_info.user),
        port=six.text_type(connection_info.port),
        identity_file=connection_info.key,
        remote_command=(['DEVSHELL_PROJECT_ID=' + project]
                        if project else []) + command_list,
        extra_flags=args.ssh_flag,
        tty=not args.command,
        options={'StrictHostKeyChecking': 'no'},
        remainder=args.ssh_args if args.ssh_args else None
    )

    if args.dry_run:
      elems = command.Build(connection_info.ssh_env)
      log.Print(' '.join([six.moves.shlex_quote(elem) for elem in elems]))
    elif args.authorize_session:
      self.done = threading.Event()
      thread = threading.Thread(target=self.Reauthorize, args=())
      thread.daemon = True
      thread.start()
      command.Run(connection_info.ssh_env)
      self.done.set()
    else:
      command.Run(connection_info.ssh_env)

  def Reauthorize(self):
    while not self.done.is_set():
      self.done.wait(
          (util.MIN_CREDS_EXPIRY - datetime.timedelta(minutes=2)).seconds)
      if not self.done.is_set():
        util.AuthorizeEnvironment()



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud domains registrations authorization-code get` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.domains import registrations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.domains import resource_args
from googlecloudsdk.command_lib.domains import util


class GetAuthorizationCode(base.DescribeCommand):
  """Get authorization code of a specific Cloud Domains registration.

  Get authorization code of a specific registration.

  You can call this API only after 60 days have elapsed since initial
  registration.

  ## EXAMPLES

  To get authorization code of ``example.com'', run:

    $ {command} example.com
  """

  @staticmethod
  def Args(parser):
    resource_args.AddRegistrationResourceArg(parser,
                                             'to get authorization code for')

  def Run(self, args):
    """Run get authorization code command."""
    api_version = registrations.GetApiVersionFromArgs(args)
    client = registrations.RegistrationsClient(api_version)
    args.registration = util.NormalizeResourceName(args.registration)
    registration_ref = args.CONCEPTS.registration.Parse()

    registration = client.Get(registration_ref)
    util.AssertRegistrationOperational(api_version, registration)

    return client.RetrieveAuthorizationCode(registration_ref)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""ai-platform models create command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.ml_engine import models
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ml_engine import constants
from googlecloudsdk.command_lib.ml_engine import endpoint_util
from googlecloudsdk.command_lib.ml_engine import flags
from googlecloudsdk.command_lib.ml_engine import models_util
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log

_REGION_FLAG_HELPTEXT = """\
Google Cloud region of the regional endpoint to use for this command.

If you specify this flag, do not specify `--regions`.

If you specify `--region=global`, the model will be deployed to 'us-central1'
by default using the global endpoint. Please use `--regions` only if you want
to change the region where the model will be deployed against the global
endpoint.

If both flags are unspecified and you don't set ``ai_platform/region'', you will
be prompted for region of the regional endpoint.

Learn more about regional endpoints and see a list of available regions:
 https://cloud.google.com/ai-platform/prediction/docs/regional-endpoints
"""


def _AddCreateArgs(parser,
                   support_console_logging=False):
  """Get arguments for the `ai-platform models create` command."""
  flags.GetModelName().AddToParser(parser)
  flags.GetDescriptionFlag('model').AddToParser(parser)
  region_group = parser.add_mutually_exclusive_group()
  region_group.add_argument(
      '--region',
      choices=constants.SUPPORTED_REGIONS_WITH_GLOBAL,
      help=_REGION_FLAG_HELPTEXT)
  region_group.add_argument(
      '--regions',
      metavar='REGION',
      type=arg_parsers.ArgList(min_length=1),
      help="""\
The Google Cloud region where the model will be deployed (currently only a
single region is supported) against the global endpoint.

If you specify this flag, do not specify `--region`.

Defaults to 'us-central1' while using the global endpoint.
""")
  parser.add_argument(
      '--enable-logging',
      action='store_true',
      help='If set, enables StackDriver Logging for online prediction. These '
           'logs are like standard server access logs, containing information '
           'such as timestamps and latency for each request.')
  if support_console_logging:
    parser.add_argument(
        '--enable-console-logging',
        action='store_true',
        help='If set, enables StackDriver Logging of stderr and stdout streams '
             'for online prediction. These logs are more verbose than the '
             'standard access logs and can be helpful for debugging.')
  labels_util.AddCreateLabelsFlags(parser)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Create a new AI Platform model."""

  @staticmethod
  def Args(parser):
    _AddCreateArgs(parser)

  def _Run(self, args, support_console_logging=False):
    region, model_regions = models_util.GetModelRegion(args)
    with endpoint_util.MlEndpointOverrides(region=region):
      models_client = models.ModelsClient()
      labels = models_util.ParseCreateLabels(models_client, args)
      enable_console_logging = (
          support_console_logging and args.enable_console_logging)
      model = models_util.Create(
          models_client,
          args.model,
          model_regions,
          enable_logging=args.enable_logging,
          enable_console_logging=enable_console_logging,
          labels=labels,
          description=args.description)
      log.CreatedResource(model.name, kind='ai platform model')

  def Run(self, args):
    self._Run(args)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class CreateBeta(Create):
  """Create a new AI Platform model."""

  @staticmethod
  def Args(parser):
    _AddCreateArgs(parser, support_console_logging=True)

  def Run(self, args):
    self._Run(args, support_console_logging=True)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing the associations of an organization or folder resource."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.firewall_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.firewall_policies import flags
import six

# TODO(b/175792396): change displayName to shortName once API is rolled out
DEFAULT_LIST_FORMAT = """\
  table(
    name,
    displayName,
    firewallPolicyId
  )"""


class List(base.DescribeCommand, base.ListCommand):
  """List the associations of an organization or folder resource.

  *{command}* is used to list the associations of an organization or folder
   resource.
  """

  @classmethod
  def Args(cls, parser):
    flags.AddArgsListAssociation(parser)
    parser.display_info.AddFormat(DEFAULT_LIST_FORMAT)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    org_firewall_policy = client.OrgFirewallPolicy(
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower())

    target_resource = None

    if args.IsSpecified('organization'):
      target_resource = 'organizations/' + args.organization

    elif args.IsSpecified('folder'):
      target_resource = 'folders/' + args.folder
    res = org_firewall_policy.ListAssociations(
        target_resource=target_resource, only_generate_request=False)
    if not res:
      return None
    return res[0].associations


List.detailed_help = {
    'EXAMPLES':
        """\
    To list the associations of the folder with ID ``987654321", run:

      $ {command} --folder=987654321
    """,
}



# Lint as: python3
# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Revoke a certificate."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import certificate_utils
from googlecloudsdk.api_lib.privateca import certificate_utils_v1
from googlecloudsdk.api_lib.privateca import request_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.privateca import flags
from googlecloudsdk.command_lib.privateca import flags_v1
from googlecloudsdk.command_lib.privateca import resource_args
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import log
from googlecloudsdk.core import resources
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.util import times


# TODO(b/177604350): Remove Beta code paths.
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class RevokeBeta(base.SilentCommand):
  r"""Revoke a certificate.

  Revokes the given certificate for the given reason.

  ## EXAMPLES

  To revoke the 'frontend-server-tls' certificate due to key compromise:

    $ {command} \
      --certificate=frontend-server-tls \
      --issuer=server-tls-1 --issuer-location=us-west1 \
      --reason=key_compromise

  To revoke the a certificate with the serial number
  '7dc1d9186372de2e1f4824abb1c4c9e5e43cbb40' due to a newer one being issued:

    $ {command} \
      --serial-number=7dc1d9186372de2e1f4824abb1c4c9e5e43cbb40 \
      --issuer=server-tls-1 --issuer-location=us-west1 \
      --reason=superseded
  """

  @staticmethod
  def Args(parser):
    id_group = parser.add_group(
        mutex=True, required=True, help='The certificate identifier.')
    base.Argument(
        '--serial-number',
        help='The serial number of the certificate.').AddToParser(id_group)
    concept_parsers.ConceptParser([
        presentation_specs.ResourcePresentationSpec(
            '--certificate',
            resource_args.CreateCertificateResourceSpec('certificate'),
            'The certificate to revoke.',
            flag_name_overrides={
                'issuer': '',
                'issuer-location': '',
                'project': '',
            },
            group=id_group),
        presentation_specs.ResourcePresentationSpec(
            '--issuer',
            resource_args.CreateCertificateAuthorityResourceSpec(
                'Issuing CA', 'issuer', 'issuer-location'),
            'The issuing certificate authority of the certificate to revoke.',
            required=False),
    ]).AddToParser(parser)

    flags.AddRevocationReasonFlag(parser)

  @staticmethod
  def ParseCertificateResource(args):
    """Gets the certificate resource to be revoked based on the specified args."""
    # Option 1: user specified full resource name for the certificate.
    cert_ref = args.CONCEPTS.certificate.Parse()
    if cert_ref:
      return cert_ref

    if not args.IsSpecified('issuer'):
      raise exceptions.RequiredArgumentException(
          '--issuer',
          ('The issuing CA is required if a full resource name is not provided '
           'for --certificate.'))

    issuer_ref = args.CONCEPTS.issuer.Parse()
    if not issuer_ref:
      raise exceptions.RequiredArgumentException('--issuer', (
          'The issuer flag is not fully specified. Please add the '
          "--issuer-location flag or specify the issuer's full resource name."))

    cert_collection_name = 'privateca.projects.locations.certificateAuthorities.certificates'
    # Option 2: user specified certificate ID + issuer.
    if args.IsSpecified('certificate'):
      return resources.REGISTRY.Parse(
          args.certificate,
          collection=cert_collection_name,
          params={
              'projectsId': issuer_ref.projectsId,
              'locationsId': issuer_ref.locationsId,
              'certificateAuthoritiesId': issuer_ref.certificateAuthoritiesId,
          })

    # Option 3: user specified serial number + issuer.
    if args.IsSpecified('serial_number'):
      certificate = certificate_utils.GetCertificateBySerialNum(
          issuer_ref, args.serial_number)
      return resources.REGISTRY.Parse(
          certificate.name, collection=cert_collection_name)

    raise exceptions.OneOfArgumentsRequiredException(
        ['--certificate', '--serial-number'],
        ('To revoke a Certificate, please provide either its resource ID or '
         'serial number.'))

  def Run(self, args):
    cert_ref = RevokeBeta.ParseCertificateResource(args)

    if not console_io.PromptContinue(
        message='You are about to revoke Certificate [{}]'.format(
            cert_ref.RelativeName()),
        default=True):
      log.status.Print('Aborted by user.')
      return

    reason = flags.ParseRevocationChoiceToEnum(args.reason)

    client = privateca_base.GetClientInstance()
    messages = privateca_base.GetMessagesModule()

    certificate = client.projects_locations_certificateAuthorities_certificates.Revoke(
        messages.
        PrivatecaProjectsLocationsCertificateAuthoritiesCertificatesRevokeRequest(
            name=cert_ref.RelativeName(),
            revokeCertificateRequest=messages.RevokeCertificateRequest(
                reason=reason, requestId=request_utils.GenerateRequestId())))

    revoke_time = times.ParseDateTime(
        certificate.revocationDetails.revocationTime)
    log.status.Print('Revoked certificate [{}] at {}.'.format(
        certificate.name, times.FormatDateTime(revoke_time,
                                               tzinfo=times.LOCAL)))


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Revoke(base.SilentCommand):
  r"""Revoke a certificate.

  Revokes the given certificate for the given reason.

  ## EXAMPLES

  To revoke the 'frontend-server-tls' certificate due to key compromise:

    $ {command} \
      --certificate=frontend-server-tls \
      --issuer-pool=my-pool --issuer-location=us-west1 \
      --reason=key_compromise

  To revoke the a certificate with the serial number
  '7dc1d9186372de2e1f4824abb1c4c9e5e43cbb40' due to a newer one being issued:

    $ {command} \
      --serial-number=7dc1d9186372de2e1f4824abb1c4c9e5e43cbb40 \
      --issuer-pool=my-pool --issuer-location=us-west1 \
      --reason=superseded
  """

  @staticmethod
  def Args(parser):
    id_group = parser.add_group(
        mutex=True, required=True, help='The certificate identifier.')
    base.Argument(
        '--serial-number',
        help='The serial number of the certificate.').AddToParser(id_group)
    concept_parsers.ConceptParser([
        presentation_specs.ResourcePresentationSpec(
            '--certificate',
            resource_args.CreateCertResourceSpec('certificate'),
            'The certificate to revoke.',
            flag_name_overrides={
                'issuer-pool': '',
                'issuer-location': '',
                'project': '',
            },
            group=id_group),
        presentation_specs.ResourcePresentationSpec(
            '--issuer-pool',
            resource_args.CreateCaPoolResourceSpec('Issuing CA pool',
                                                   'issuer-location'),
            'The issuing CA pool of the certificate to revoke.',
            required=False),
    ]).AddToParser(parser)

    flags_v1.AddRevocationReasonFlag(parser)

  @staticmethod
  def ParseCertificateResource(args):
    """Gets the certificate resource to be revoked based on the specified args."""
    # Option 1: user specified full resource name for the certificate.
    cert_ref = args.CONCEPTS.certificate.Parse()
    if cert_ref:
      return cert_ref

    if not args.IsSpecified('issuer_pool'):
      raise exceptions.RequiredArgumentException('--issuer-pool', (
          'The issuing CA pool is required if a full resource name is not provided '
          'for --certificate.'))

    issuer_ref = args.CONCEPTS.issuer_pool.Parse()
    if not issuer_ref:
      raise exceptions.RequiredArgumentException('--issuer-pool', (
          'The issuer flag is not fully specified. Please add the '
          "--issuer-location flag or specify the issuer's full resource name."))

    cert_collection_name = 'privateca.projects.locations.caPools.certificates'
    # Option 2: user specified certificate ID + issuer.
    if args.IsSpecified('certificate'):
      return resources.REGISTRY.Parse(
          args.certificate,
          collection=cert_collection_name,
          params={
              'projectsId': issuer_ref.projectsId,
              'locationsId': issuer_ref.locationsId,
              'caPoolsId': issuer_ref.caPoolsId,
          })

    # Option 3: user specified serial number + issuer.
    if args.IsSpecified('serial_number'):
      certificate = certificate_utils_v1.GetCertificateBySerialNum(
          issuer_ref, args.serial_number)
      return resources.REGISTRY.Parse(
          certificate.name, collection=cert_collection_name)

    raise exceptions.OneOfArgumentsRequiredException(
        ['--certificate', '--serial-number'],
        ('To revoke a Certificate, please provide either its resource ID or '
         'serial number.'))

  def Run(self, args):
    cert_ref = Revoke.ParseCertificateResource(args)

    if not console_io.PromptContinue(
        message='You are about to revoke Certificate [{}]'.format(
            cert_ref.RelativeName()),
        default=True):
      log.status.Print('Aborted by user.')
      return

    reason = flags_v1.ParseRevocationChoiceToEnum(args.reason)

    client = privateca_base.GetClientInstance(api_version='v1')
    messages = privateca_base.GetMessagesModule(api_version='v1')

    certificate = client.projects_locations_caPools_certificates.Revoke(
        messages.PrivatecaProjectsLocationsCaPoolsCertificatesRevokeRequest(
            name=cert_ref.RelativeName(),
            revokeCertificateRequest=messages.RevokeCertificateRequest(
                reason=reason, requestId=request_utils.GenerateRequestId())))

    revoke_time = times.ParseDateTime(
        certificate.revocationDetails.revocationTime)
    log.status.Print('Revoked certificate [{}] at {}.'.format(
        certificate.name, times.FormatDateTime(revoke_time,
                                               tzinfo=times.LOCAL)))



# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This package flattens image metadata into a single tarball."""

from __future__ import absolute_import

from __future__ import print_function

import argparse
import logging
import tarfile

from containerregistry.client.v2_2 import docker_image as v2_2_image
from containerregistry.tools import logging_setup
from six.moves import zip  # pylint: disable=redefined-builtin

parser = argparse.ArgumentParser(description='Flatten container images.')

# The name of this flag was chosen for compatibility with docker_pusher.py
parser.add_argument(
    '--tarball', action='store', help='An optional legacy base image tarball.')

parser.add_argument(
    '--config',
    action='store',
    help='The path to the file storing the image config.')

parser.add_argument(
    '--digest',
    action='append',
    help='The list of layer digest filenames in order.')

parser.add_argument(
    '--layer',
    action='append',
    help='The list of compressed layer filenames in order.')

parser.add_argument(
    '--uncompressed_layer',
    action='append',
    help='The list of uncompressed layer filenames in order.')

parser.add_argument(
    '--diff_id', action='append', help='The list of diff_ids in order.')

# Output arguments.
parser.add_argument(
    '--filesystem',
    action='store',
    help='The name of where to write the filesystem tarball.')

parser.add_argument(
    '--metadata',
    action='store',
    help=('The name of where to write the container '
          'startup metadata.'))


def main():
  logging_setup.DefineCommandLineArgs(parser)
  args = parser.parse_args()
  logging_setup.Init(args=args)

  # If config is specified, use that.  Otherwise, fall back on reading
  # the config from the tarball.
  if args.config:
    logging.info('Reading config from %r', args.config)
    with open(args.config, 'r') as reader:
      config = reader.read()
  elif args.tarball:
    logging.info('Reading config from tarball %r', args.tarball)
    with v2_2_image.FromTarball(args.tarball) as base:
      config = base.config_file()
  else:
    config = args.config

  layers = list(zip(args.digest or [], args.layer or []))
  uncompressed_layers = list(
      zip(args.diff_id or [], args.uncompressed_layer or []))
  logging.info('Loading v2.2 image From Disk ...')
  with v2_2_image.FromDisk(
      config_file=config,
      layers=layers,
      uncompressed_layers=uncompressed_layers,
      legacy_base=args.tarball) as v2_2_img:
    with tarfile.open(args.filesystem, 'w:', encoding='utf-8') as tar:
      v2_2_image.extract(v2_2_img, tar)

    with open(args.metadata, 'w') as f:
      f.write(v2_2_img.config_file())


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing network firewall policies."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.network_firewall_policies import client
from googlecloudsdk.api_lib.compute.network_firewall_policies import region_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.network_firewall_policies import flags


class Describe(base.DescribeCommand):
  """Describe a Compute Engine network firewall policy.

  *{command}* is used to describe network firewall policies. A
  network firewall policy is a set of rules that controls access to various
  resources.
  """

  NETWORK_FIREWALL_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.NETWORK_FIREWALL_POLICY_ARG = flags.NetworkFirewallPolicyArgument(
        required=True, operation='get')
    cls.NETWORK_FIREWALL_POLICY_ARG.AddArgument(parser, operation_type='get')

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.NETWORK_FIREWALL_POLICY_ARG.ResolveAsResource(
        args, holder.resources)

    network_firewall_policy = client.NetworkFirewallPolicy(
        ref, compute_client=holder.client)
    if hasattr(ref, 'region'):
      network_firewall_policy = region_client.RegionNetworkFirewallPolicy(
          ref, compute_client=holder.client)

    return network_firewall_policy.Describe(only_generate_request=False)


Describe.detailed_help = {
    'EXAMPLES':
        """\
    To describe a global network firewall policy with name ``my-policy'', run:

      $ {command} my-policy --global

    To describe a regional network firewall policy with name ``my-policy'',
    in region ``my-region'', run:

      $ {command} my-policy --region=my-region
    """,
}



#!/usr/bin/env python
#
# Copyright 2015 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Top-level imports for apitools base files."""

# pylint:disable=wildcard-import
# pylint:disable=redefined-builtin
from apitools.base.py.base_api import *
from apitools.base.py.batch import *
from apitools.base.py.credentials_lib import *
from apitools.base.py.encoding import *
from apitools.base.py.exceptions import *
from apitools.base.py.extra_types import *
from apitools.base.py.http_wrapper import *
from apitools.base.py.list_pager import *
from apitools.base.py.transfer import *
from apitools.base.py.util import *

try:
    # pylint:disable=no-name-in-module
    from apitools.base.py.internal import *
except ImportError:
    pass
